#!/usr/bin/perl

# Copyright (c) 2003, Mark Daniel Reidel
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer
#       in the documentation and/or other materials provided with the
#       distribution.
#     * Neither the name of Mark Daniel Reidel nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package DirNode;

use strict;
$^W=1;
use MP3::Info;
use Ogg::Vorbis::Header;
use Digest::MD5;
use DBI;

sub new
{
	my $class = shift;

	my $self = {};

	bless $self, $class;

	return $self;
}

sub GetUseVT100 { my $self = shift; return $self->GetParent() ? $self->GetParent()->GetUseVT100() : ($self->{'VT100'} || 0); }
sub SetUseVT100 { my $self = shift; return $self->{'VT100'} = shift; }

sub GetSkipDirs { my $self = shift; return $self->GetParent() ? $self->GetParent()->GetSkipDirs() : ($self->{'SkipDirs'} || {}); }
sub SetSkipDirs { my $self = shift; return $self->{'SkipDirs'} = shift; }

sub GetDBName { my $self = shift; return $self->GetParent() ? $self->GetParent()->GetDBName() : $self->{'DB_NAME'}; }
sub SetDBName { my $self = shift; return $self->{'DB_NAME'} = shift; }

sub GetDBUser { my $self = shift; return $self->GetParent() ? $self->GetParent()->GetDBUser() : $self->{'DB_USER'}; }
sub SetDBUser { my $self = shift; return $self->{'DB_USER'} = shift; }

sub GetDBPassword { my $self = shift; return $self->GetParent() ? $self->GetParent()->GetDBPassword() : $self->{'DB_PASSWORD'}; }
sub SetDBPassword { my $self = shift; return $self->{'DB_PASSWORD'} = shift; }

sub GetDBTable { my $self = shift; return $self->GetParent() ? $self->GetParent()->GetDBTable() : $self->{'DB_TABLE'}; }
sub SetDBTable { my $self = shift; return $self->{'DB_TABLE'} = shift; }

sub GetDBSequence { my $self = shift; return $self->GetParent() ? $self->GetParent()->GetDBSequence() : $self->{'DB_SEQUENCE'}; }
sub SetDBSequence { my $self = shift; return $self->{'DB_SEQUENCE'} = shift; }

sub GetDBHandle { my $self = shift; return $self->{'DBH'}; }
sub SetDBHandle { my $self = shift; return $self->{'DBH'} = shift; }

sub DisconnectDB {
	my $self = shift;
	my $DBH = $self->GetDBHandle();

	if ( $DBH ) {
		$DBH->disconnect();
	}
	return;
}

sub GetDir
{
	my $self = shift;

	my $Dir = $self->GetBaseDir()."/".$self->GetRelDir();

	$Dir =~ s/\/+$//og;

	return $Dir;
}

sub GetDIRCount { my $self = shift; return $self->{'DirectoryCount'} || 0; }
sub SetDIRCount { my $self = shift; return $self->{'DirectoryCount'} = shift; }

sub GetBaseDir { my $self = shift; return $self->{'BaseDirectory'}; }
sub SetBaseDir
{
	my $self = shift;

	my $BaseDir = shift;
	$BaseDir =~ s/\/+$//og;

	return $self->{'BaseDirectory'} = $BaseDir;
}

sub GetRelDir { my $self = shift; return $self->{'RelativeDirectory'} || ''; }
sub SetRelDir
{
	my $self = shift;
	my $Dir = shift;

	$Dir = substr($Dir, 1) if substr($Dir, 0, 1) eq "/";

	return $self->{'RelativeDirectory'} = $Dir;
}

sub GetDescriptiveDir { my $self = shift; return $self->{'DescriptiveDirectory'}; }
sub SetDescriptiveDir { my $self = shift; return $self->{'DescriptiveDirectory'} = shift; }

sub GetDescriptiveFullDir { my $self = shift; return $self->{'DescriptiveFullDirectory'} || $self->GetDescriptiveDir(); }
sub SetDescriptiveFullDir { my $self = shift; return $self->{'DescriptiveFullDirectory'} = shift; }

sub GetParent { my $self = shift; return $self->{'Parent'}; }
sub SetParent { my $self = shift; return $self->{'Parent'} = shift; }

sub GetCurrentFile { my $self = shift; return $self->{'CurrentFile'}; }
sub SetCurrentFile { my $self = shift; return $self->{'CurrentFile'} = shift; }

sub GetCurrentFileNum { my $self = shift; return $self->{'CurrentFileNum'} ? $self->{'CurrentFileNum'} : 0; }
sub SetCurrentFileNum { my $self = shift; return $self->{'CurrentFileNum'} = shift; }

sub GetFileSizes { my $self = shift; return $self->{'FileSizes'} || 0; }
sub SetFileSizes { my $self = shift; return $self->{'FileSizes'} = shift; }

sub GetCurrentFullFile { my $self = shift; return $self->{'CurrentFullFile'}; }
sub SetCurrentFullFile { my $self = shift; return $self->{'CurrentFullFile'} = shift; }

#########################################################################################

sub GetDirectNodeGraph
{
	my $self = shift;

	my $PreviousNodeGraph = '';
	my $Parent = $self->GetParent();
	$PreviousNodeGraph = $Parent->GetNodeGraph() if $Parent;
	if ( $self->GetCurrentFileNum() == $self->GetDIRCount() )
	{
		if ( $self->GetUseVT100() ) {
			return $PreviousNodeGraph . "    (B)0mqqq ";
		} else {
			return $PreviousNodeGraph . "   `-";
		}
	}
	if ( $self->GetUseVT100() ) {
		return $PreviousNodeGraph . "   (B)0 tqqq ";
	} else {
		return $PreviousNodeGraph . "   |-";
	}
}

sub GetNodeGraph
{
	my $self = shift;

	my $PreviousNodeGraph = '';
	my $Parent = $self->GetParent();
	$PreviousNodeGraph = $Parent->GetNodeGraph() if $Parent;
	if ( $self->GetCurrentFileNum() == $self->GetDIRCount() )
	{
		return $PreviousNodeGraph . "    ";
	}
	if ( $self->GetUseVT100() ) {
		return $PreviousNodeGraph . "   (B)0 x";
	} else {
		return $PreviousNodeGraph . "   |";
	}
}

sub InsertIntoDB {
	my $self = shift;

	my $path = shift;
	my $file = shift;
	my $size = shift;
	my $length = shift;
	my $bitrate = shift;
	my $samplerate = shift;
	my $channels = shift;
	my $vbr = shift;

	my $sql;
	my $STH;

	my $DBH = $self->GetDBHandle();
	if ( !$DBH ) {
		$DBH = DBI->connect("DBI:Pg:dbname=".$self->GetDBName(),
		                    $self->GetDBUser(),$self->GetDBPassword());
		if ( !$DBH ) {
			die("Unable to connect to database, giving up.\n");
		}
		$self->SetDBHandle($DBH);
	}

	$sql = "SELECT NEXTVAL('".$self->GetDBSequence()."')";
	$STH = $DBH->prepare($sql) or die("Error executing Query: $DBI::errstr\n");
	$STH->execute() or die("Error executing Query: $DBI::errstr\n");
	(my $sid) = $STH->fetchrow_array();
	
	$sql = "INSERT INTO ".$self->GetDBTable()." (sid, path, file, size, length, bitrate, samplerate, channels, vbr) VALUES (".
	       "$sid, ".($DBH->quote($path)).", ".($DBH->quote($file)).", '$size', '".(int($length))."', '".($bitrate*1000)."', '".($samplerate*1000)."', '$channels', '$vbr')";
	$STH = $DBH->prepare($sql) or die("Error executing Query: $DBI::errstr\n");
	$STH->execute() or die("Error executing Query: $DBI::errstr\n");
}

sub GetOggDescription
{
	my $self = shift;
	my $Filename = shift;

	my $Ogg = Ogg::Vorbis::Header->new($Filename);

	my $Info      = $Ogg->info();
	my $Channels  = $Info->{'channels'};
	my $Bitrate   = int($Info->{'bitrate_nominal'} / 1000);
	my $Frequency = int($Info->{'rate'} / 1000);

	my $Result = [$Info->{'length'}, 1, $Bitrate, $Frequency, $Channels];
	undef $Ogg;
	return $Result;
}

sub GetMP3Description
{
	my $self = shift;
	my $Filename = shift;

	my $data = MP3::Info::get_mp3info($Filename) or die "Unable to open $Filename: $!\n";

	my $Hours = int($data->{'MM'} / 60);
	my $Minutes = $data->{'MM'} % 60;
	my $Seconds = $data->{'SS'};
	my $VBR = $data->{'VBR'};
	my $Bitrate = $data->{'BITRATE'};
	my $Frequency = $data->{'FREQUENCY'};
	my $Stereo = $data->{'STEREO'};
	my $Result = [$data->{'MM'} * 60 + $data->{'SS'}, $data->{'VBR'}, $Bitrate, $Frequency, $Stereo ? 2 : 1];
	undef $data;
	return $Result;
}

sub GetDescription
{
	my $self = shift;
	my $Filename = shift;

	if ( substr(lc($Filename), -4) eq ".ogg" ) {
		return $self->GetOggDescription($Filename);
	} elsif ( substr(lc($Filename), -4) eq ".mp3") {
		return $self->GetMP3Description($Filename);
	} else {
		return "???";
	}
}

sub GetFileLine
{
	my $self = shift;

	my $FileInfo = $self->GetDescription($self->GetCurrentFullFile());
	$self->InsertIntoDB($self->GetDir(),$self->GetCurrentFile(),(-s $self->GetCurrentFullFile()), $FileInfo->[0], $FileInfo->[2], $FileInfo->[3], $FileInfo->[4], $FileInfo->[1]);
	return 1;
}

sub Macro_filedescriptions
{
	my $self = shift;

	my $Directory = $self->GetDir();
	my $File;
	my $FullFile;

	my $HTML = '';
	my $Count = 0;

	my $FileSizes = 0;

	my @DIRs = ();
	my @Files = ();

	opendir(DIR, $Directory."\/") or die("Unable to open directory $Directory for browsing: $!\n");
	my @DirContent = readdir(DIR);
	closedir(DIR);

	foreach $File (sort {lc($a) cmp lc($b)} @DirContent)
	{
		next if ( ($File eq '.') || ($File eq '..') );
		$FullFile = $Directory."\/".$File;
		if (-d $FullFile)
		{
			next if $self->GetSkipDirs()->{$FullFile};
			next if $self->GetSkipDirs()->{$File};
			push(@DIRs, [$File, $FullFile]);
		}
		else
		{
			next if $File =~ /^\[/o;
			next unless $File =~ /\.(mp3|ogg)$/oi;
			push(@Files, [$File, $FullFile]);
		}
	}
	
	if ( @DIRs )
	{
		$self->SetDIRCount(scalar(@DIRs));
		foreach $File (@DIRs)
		{
			$self->SetCurrentFileNum(++$Count);
			my $Node = DirNode->new();
			$Node->SetBaseDir($self->GetBaseDir());
			$Node->SetDescriptiveDir($File->[0]);
			$Node->SetRelDir($self->GetRelDir()."\/".$File->[0]);
			$Node->SetDescriptiveFullDir($self->GetDescriptiveFullDir()." -> ".$File->[0]);
			$Node->SetParent($self);
			$Node->Run().
			$self->SetFileSizes($self->GetFileSizes() + $Node->GetFileSizes());
			$Node->DisconnectDB();
			undef $Node;
		}
	}

	if ( @Files )
	{
		$Count = 0;

		foreach $File (@Files)
		{
			if ( -e $File->[1] ) {
				$self->SetCurrentFileNum(++$Count);
				$self->SetCurrentFile($File->[0]);
				$self->SetCurrentFullFile($File->[1]);
				$self->GetFileLine();
				$self->SetFileSizes($self->GetFileSizes() + (-s $File->[1]));
				$FileSizes += (-s $File->[1]);
			}
		}
	}

	return 1;
}

sub Run
{
	my $self = shift;

	print $self->GetParent()->GetDirectNodeGraph() if $self->GetParent();
	print $self->GetDescriptiveDir()."\n";
    $self->Macro_filedescriptions();
	return 1;
}

1;

use strict;
$^W=1;

my $DBH;
my $STH;
my $sql;
my $Config = {};
my $line;
my $linenr = 0;
my @parts;

open(CFG, "/usr/local/etc/musicindexer.cfg") or (open(CFG, "/etc/musicindexer.cfg") or die("Unable to open the config file: $!\n"));
while ($line = <CFG>) {
	chomp($line);
	$linenr++;
	next if ($line =~ /^\s*$/);
	next if ($line =~ /^#/);
	if ( $line !~ /\+?=/ ) {
		die("Error in Configfile line $linenr: $line");
	}
	@parts = split(/\s*\+?=\s*/, $line);
	if ( $line =~ /\+=/ ) {
		if ( !defined(@{$Config->{$parts[0]}}) ) {
			$Config->{$parts[0]} = [$Config->{$parts[0]}];
		}
		push(@{$Config->{$parts[0]}}, $parts[1]);
	} else {
		$Config->{$parts[0]} = $parts[1];
	}
}
close(CFG);

$DBH = DBI->connect("DBI:Pg:dbname=".$Config->{'Database'}, $Config->{'Username'}, $Config->{'Password'});
if ( !$DBH ) {
	die("Unable to connect to database, giving up.\n");
}

$sql = "DROP TABLE ".$Config->{'Table'};
$STH = $DBH->prepare($sql) or die("Error executing Query: $DBI::errstr\n");
$STH->execute() or die("Error executing Query: $DBI::errstr\n");
$STH->finish();

$sql = "SELECT SETVAL('".$Config->{'Sequence'}."',1)";
$STH = $DBH->prepare($sql) or die("Error executing Query: $DBI::errstr\n");
$STH->execute() or die("Error executing Query: $DBI::errstr\n");
$STH->finish();

$sql = 
"CREATE TABLE ".$Config->{'Table'}."(".
"  sid        INT PRIMARY KEY,".
"  path       VARCHAR(400) NOT NULL,".
"  file       VARCHAR(400) NOT NULL,".
"  size       INT NOT NULL,".
"  length     INT,".
"  bitrate    INT,".
"  samplerate FLOAT,".
"  channels   INT,".
"  vbr        BOOL".
")";
$STH = $DBH->prepare($sql) or die("Error executing Query: $DBI::errstr\n");
$STH->execute() or die("Error executing Query: $DBI::errstr\n");
$STH->finish();

$sql = "CREATE INDEX ".$Config->{'Table'}."_file_idx ON ".$Config->{'Table'}."(file)";
$STH = $DBH->prepare($sql) or die("Error executing Query: $DBI::errstr\n");
$STH->execute() or die("Error executing Query: $DBI::errstr\n");
$STH->finish();

$sql = "CREATE INDEX ".$Config->{'Table'}."_path_idx ON ".$Config->{'Table'}."(path)";
$STH = $DBH->prepare($sql) or die("Error executing Query: $DBI::errstr\n");
$STH->execute() or die("Error executing Query: $DBI::errstr\n");
$STH->finish();

$DBH->disconnect();

my $Node = DirNode->new();
$Node->SetDBName($Config->{'Database'});
$Node->SetDBUser($Config->{'Username'});
$Node->SetDBPassword($Config->{'Password'});
$Node->SetDBTable($Config->{'Table'});
$Node->SetDBSequence($Config->{'Sequence'});
$Node->SetBaseDir($Config->{'Basedir'});
$Node->SetUseVT100($Config->{'UseVT100'});
$Node->SetDescriptiveDir($Config->{'Basedescription'});
my $Skipdirs = {};
if ( defined($Config->{'Skipdir'}) ) {
	if ( !defined(@{$Config->{'Skipdir'}}) ) {
		$Config->{'Skipdir'} = [$Config->{'Skipdir'}];
	}
	foreach (@{$Config->{'Skipdir'}}) {
		$Skipdirs->{$_} = 1;
	}
	$Node->SetSkipDirs( $Skipdirs );
}

$Node->Run();
