<?

/*
 * Copyright (c) 2003, Mark Daniel Reidel
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer
 *       in the documentation and/or other materials provided with the
 *       distribution.
 *     * Neither the name of Mark Daniel Reidel nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

set_time_limit(1000);

include("Smarty.class.php");

$Smarty = new Smarty;

$Smarty->register_function("qualitycolor","smarty_function_qualitycolor");
$Smarty->register_modifier("bitrate",     "smarty_modifier_bitrate");
$Smarty->register_modifier("samplerate",  "smarty_modifier_samplerate");
$Smarty->register_modifier("secstolength","smarty_modifier_secstolength");

$Smarty->assign("action", $_REQUEST["action"]);
$Smarty->assign("Config", GetConfig());


//  START SMARTY PLUGINS  ///////////////////////////////////////////////////

function smarty_function_qualitycolor($params, &$smarty)
{
	$factor = $params['bitrate'] / 128000 + $params['samplerate'] / 44000;
	return ($factor >= 2)
	? 'green'
	: (($factor >= 1)
		? '#ff9e00'
		: 'red');
}

function smarty_modifier_bitrate($length)
{
	return sprintf("%d kBit", $length / 1000);
}

function smarty_modifier_samplerate($length)
{
	return sprintf("%.1f kHz", $length / 1000);
}

function smarty_modifier_secstolength($length)
{
	$result = "";
	if ( $length > 3600 ) {
		$hours = $length / 3600;
		$result = sprintf("%d:", $hours);
	}
	return sprintf("$result%02d:%02d", ($length % 3600)/60, $length % 60);
}

//  END SMARTY PLUGINS  /////////////////////////////////////////////////////

function Error($err) {
	echo("Error: $err<br>\n");
	die();
}

function AddError($msg) {
	$GLOBALS["Smarty"]->append("Messages", array($msg, "error"));
}

function AddWarning($msg) {
	$GLOBALS["Smarty"]->append("Messages", array($msg, "warning"));
}

function DBConnect() {
	$connectstring = "dbname=".GetConfig("Database")." user=".GetConfig("Username")." password=".GetConfig("Password");
	if( !($mydbh = @pg_Connect($connectstring)) )  {
		Error("Unable to connect to database *cries sadly*");
	}
	return $mydbh;
}

function GetConfig($search=0) {
	if ( !$GLOBALS["CONFIG"] ) {
		$GLOBALS["CONFIG"] = array();
		if (   !($file = @fopen("/etc/musicindexer.cfg", "r"))
		    && !($file = @fopen("/usr/local/etc/musicindexer.cfg", "r")) ) {
			Error("Unable to open configfile");
		}
		$linenr = 0;
		while ( $line = fgets($file) ) {
			$line = trim($line);
			$linenr++;
			if (   !preg_match("/^\s*$/", $line)
			    && !preg_match("/^#/", $line) ) {
				if ( !preg_match("/=/", $line) ) {
					Error("Error in Configfile line $linenr: $line");
				}
				list($key, $value) = preg_split("/\s*=\s*/", $line);
				$GLOBALS["CONFIG"][$key] = $value;
			}
		}
		fclose($file);
	}
	return $search ? $GLOBALS["CONFIG"][$search] : $GLOBALS["CONFIG"];
}

function DBQuery($sql, $Error) {
	if ( !$GLOBALS["DBH"] ) {
		$GLOBALS["DBH"] = DBConnect();
	}
	$ret = @pg_exec($GLOBALS["DBH"], $sql);
	if ( !$ret ) {
		Error("$Error<br>\n Query: $sql<br>\nError: ".pg_errormessage($GLOBALS["DBH"]));
	}

	return $ret;
}

function DBEscape($str) {
	return ini_get("magic_quotes_gpc") ? $str : pg_escape_string($str);
}

function IsForbiddenDir($Dir) {
	$RegDir = preg_quote(GetConfig("Basedir"), "/");
	if (   !preg_match("/^$RegDir(\/|$)/", $Dir)
	    || !file_exists(realpath($Dir)) ) {
		AddError("The directory \"".stripslashes($Dir)."\" does not exist.");
		return 1;
	}
	return 0;
}

function BrowseDir($Dir) {
	if ( IsForbiddenDir($Dir) ) {
		return 0;
	}
	$Dir = realpath($Dir);
	$sql = "SELECT SUM(size) AS size, SUM(length) AS length, ".
               "SUBSTRING(path FROM '^(".DBEscape($Dir)."/[^/]+)($|/)') AS fake_path ".
               "FROM music WHERE path LIKE '".DBEscape($Dir)."/%' ".
               "GROUP BY fake_path ORDER BY fake_path ASC";
	$ret = DBQuery($sql, "Fehler beim Auslesen der Verzeichnisse");
	$ParentDir = preg_replace("/^(.*)\/.*$/", "$1", $Dir);

	$FullPath = array();
	$CutDir = preg_replace("/^".preg_quote(GetConfig("Basedir"), "/")."/", "", $Dir);
	array_push($FullPath, array("short" => GetConfig("Basedescription"), "full" => GetConfig("Basedir")));
	
	$Parts = split("/", $CutDir);
	$tmp = GetConfig("Basedir");
	foreach ($Parts as $Part) {
		if ( $Part != "" ) {
			$tmp .= "/$Part";
			array_push($FullPath, array("short" => $Part, "full" => $tmp));
		}
	}
	$GLOBALS["Smarty"]->assign("FullPath", $FullPath);

	if ( $Dir != GetConfig("Basedir") ) {
		$Verzeichnisse = array( array("size" => 0, "fake_path" => $ParentDir, "short_path" => array("fake_path" => "..")));
	} else {
		$Verzeichnisse = array();
	}
	for ($i = 0; $i < pg_numrows($ret); $i++) {
		$Verzeichnis = pg_fetch_array($ret, $i);
		$Verzeichnis["size"] /= (1024*1024);
		$Verzeichnis["short_path"] = preg_replace("/^.*\/([^\/]+)$/", "$1", $Verzeichnis);
		array_push($Verzeichnisse, $Verzeichnis);
	}
	$GLOBALS["Smarty"]->assign("Verzeichnisse", $Verzeichnisse);
	$sql = "SELECT * FROM music WHERE path='".DBEscape($Dir)."' ORDER BY file ASC";
	$ret = DBQuery($sql, "Error reading all songs in directory \"$Dir\".");
	$Songs = array();
	for ($i = 0; $i < pg_numrows($ret); $i++) {
		$Song = pg_fetch_array($ret, $i);
		$Song["size"] /= (1024*1024);
		array_push($Songs, $Song);
	}
	$GLOBALS["Smarty"]->assign("Songs", $Songs);
	return 1;
}

function ShowSearchPage() {
	$sql = "SELECT COUNT(*), SUM(size) FROM music";
	$ret = DBQuery($sql, "Error while counting the total size.");
	list($TotalSongs, $TotalSize) = pg_fetch_row($ret, 0);
	$TotalSize /= (1024*1024*1024);
	$GLOBALS["Smarty"]->assign("TotalSongs", $TotalSongs);
	$GLOBALS["Smarty"]->assign("TotalSize", $TotalSize);
	$GLOBALS["Smarty"]->display("musicsearch.tpl");
}

function ShowResultPage($Search) {
	$GLOBALS["Smarty"]->assign("Search", ini_get("magic_quotes_gpc") ? stripslashes($Search) : $Search );
	$Tokens = split(" ", $Search);
	$SearchTokens = array();
	foreach ($Tokens as $Token) {
		array_push($SearchTokens, "(path ILIKE '%".DBEscape($Token)."%' OR ".
		                          "file ILIKE '%".DBEscape($Token)."%')");
	}
	$sql = "SELECT * FROM music WHERE ".join(" AND ", $SearchTokens)." ORDER BY path ASC,file ASC, sid ASC";
	if ( GetConfig("Maxresults") > 0 ) {
		$sql .= " LIMIT ".GetConfig("Maxresults");
	}
	
	$ret = DBQuery($sql, "Error doing search.");
	$Songs = array();
	for ($i = 0; $i < pg_numrows($ret); $i++) {
		$Song = pg_fetch_array($ret, $i);
		$Song["size"] /= (1024*1024);
		array_push($Songs, $Song);
	}
	$GLOBALS["Smarty"]->assign("Songs", $Songs);
	return 1;
}

function DownloadSong($sid) {
	$sql = "SELECT path,file FROM music WHERE sid='".DBEscape($sid)."'";
	$ret = DBQuery($sql, "Error searching song nr. $sid.");
	if ( !pg_numrows($ret) ) {
		AddError("Song nr. $sid does not exist!");
		return 0;
	}
	$Song = pg_fetch_array($ret, 0);

	header("Content-disposition: attachment; filename=\"".$Song["file"]."\"");
	header("Content-Length: ".filesize($Song["path"]."/".$Song["file"]));
	header("Content-type: application/octetstream");
	header("Pragma: ");
	header("Cache-Control: cache");
	header("Expires: 0");

	@readfile($Song["path"]."/".$Song["file"]);
	return 1;
}

function DownloadDir($Dir) {
	if ( !GetConfig("Allowtar") ) {
		AddError("This function is not enabled on this server.");
		return 0;
	}
	if ( IsForbiddenDir($Dir) ) {
		return 0;
	}

	$Parts = split("/", realpath(ini_get("magic_quotes_gpc") ? stripslashes($Dir) : $Dir));
	$RelDir = escapeshellarg(array_pop($Parts));

	header("Content-disposition: attachment; filename=\"$RelDir.tar\"");
	header("Content-type: application/octetstream");
	header("Pragma: ");
	header("Cache-Control: cache");
	header("Expires: 0");

	$handle = @popen("/usr/bin/tar -C '".escapeshellarg(join("/", $Parts))."' -c -f - '$RelDir'", "r");
	while ( !feof($handle) ) {
		print(fread($handle, 16384));
		flush();
	}
	pclose($handle);
	return 1;
}

function tryShowResultPage($key) {
	ShowResultPage($key);
	tryShowSearchPage();
}

function tryDownloadSong($sid) {
	if ( !DownloadSong($sid) ) {
		tryShowSearchPage();
	}
}

function tryDownloadDir($dir) {
	if ( !DownloadDir($dir) ) {
		tryShowSearchPage();
	}
}

function tryBrowseDir($dir) {
	BrowseDir($dir);
	tryShowSearchPage();
}

function tryShowSearchPage() {
	ShowSearchPage();
}

switch($_REQUEST["action"]) {
  case "search": tryShowResultPage($_REQUEST["key"]); break;
  case "download": tryDownloadSong($_REQUEST["sid"]); break;
  case "downloaddir": tryDownloadDir($_REQUEST["dir"]); break;
  case "browse": tryBrowseDir($_REQUEST["dir"]); break;
  default: tryShowSearchPage(); break;
}

?>
